@@ -25,12 +25,13 @@
 #include <netinet/in.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include <string>
 
-#include <libcapsinetwork/socket.h>
-
-#include "const.h"
+#include "config.h"
+#include "socket.h"
 #include "auction.h"
 #include "event.h"
 #include "game.h"
@@ -40,6 +41,10 @@
 #include "player.h"
 #include "server.h"
 
+#if USE_SYSTEMD_DAEMON
+#include <systemd/sd-daemon.h>
+#endif /* USE_SYSTEMD_DAEMON */
+
 MonopdServer::MonopdServer() : GameObject(0)
 {
 	m_nextGameId = m_nextPlayerId = 1;
@@ -73,7 +78,7 @@ Event *MonopdServer::newEvent(unsigned int eventType, Game *game, int id)
 {
 	Event *newEvent = new Event(id, (Event::EventType)eventType, game);
 	m_events.push_back(newEvent);
-	printf("newEvent %d/%d\n", id, m_events.size());
+	printf("newEvent %d/%d\n", id, (int)m_events.size());
 	return newEvent;
 }
 
@@ -82,7 +87,7 @@ void MonopdServer::delEvent(Event *event)
 	for(std::vector<Event *>::iterator it = m_events.begin(); it != m_events.end() && (*it) ; ++it)
 		if (*it == event)
 		{
-			printf("delEvent %d/%d\n", event->id(), m_events.size()-1);
+			printf("delEvent %d/%d\n", event->id(), (int)m_events.size()-1);
 			delete event;
 			m_events.erase(it);
 			break;
@@ -133,12 +138,14 @@ void MonopdServer::newGame(Player *player, const std::string gameType)
 		return;
 	}
 
-	syslog( LOG_INFO, "new game: id=[%d], type=[%s], games=[%d]", game->id(), gameType.c_str(), m_games.size() );
+	syslog( LOG_INFO, "new game: id=[%d], type=[%s], games=[%d]", game->id(), gameType.c_str(), (int)m_games.size() );
 
 	game->addPlayer(player, true);
 
 	// FIXME: DEPRECATED 1.0
 	ioWrite(std::string("<monopd><updategamelist type=\"add\"><game id=\"") + itoa(game->id()) + "\" players=\"1\" gametype=\"" + game->gameType() + "\" name=\"" + game->name() + "\" description=\"" + game->getStringProperty("description") + "\" canbejoined=\"" + itoa(game->getBoolProperty("canbejoined")) + "\"/></updategamelist></monopd>\n");
+
+	updateSystemdStatus();
 }
 
 void MonopdServer::joinGame(Player *pInput, unsigned int gameId, const bool &spectator)
@@ -146,7 +153,7 @@ void MonopdServer::joinGame(Player *pInput, unsigned int gameId, const bool &spe
 	Game *game = findGame(gameId);
 	if (!game)
 	{
-		pInput->ioError("There is no game with id %ld.", gameId);
+		pInput->ioError("There is no game with id %d.", gameId);
 		return;
 	}
 
@@ -159,7 +166,7 @@ void MonopdServer::joinGame(Player *pInput, unsigned int gameId, const bool &spe
 			game->ioInfo("%s joins as spectator.", pInput->name().c_str());
 		}
 		else
-			pInput->ioError("Game %ld doesn't allow spectators.", gameId);
+			pInput->ioError("Game %d doesn't allow spectators.", gameId);
 		return;
 	}
 
@@ -194,7 +201,7 @@ void MonopdServer::exitGame(Game *game, Player *pInput)
 	sendGameList(pInput);
 }
 
-Game *MonopdServer::findGame(unsigned int gameId)
+Game *MonopdServer::findGame(int gameId)
 {
 	Game *game = 0;
 	for(std::vector<Game *>::iterator it = m_games.begin(); it != m_games.end() && (game = *it) ; ++it)
@@ -223,7 +230,7 @@ void MonopdServer::delGame(Game *game, bool verbose)
 	}
 
 	// Remove events from game
-	for(std::vector<Event *>::iterator it = m_events.begin(); it != m_events.end() && (*it) ; ++it)
+	for(std::vector<Event *>::iterator it = m_events.begin(); it != m_events.end() && (*it) ; )
 	{
 		if ( (*it)->game() == game )
 		{
@@ -231,6 +238,7 @@ void MonopdServer::delGame(Game *game, bool verbose)
 			it = m_events.begin();
 			continue;
 		}
+		++it;
 	}
 
 	// Remove game
@@ -240,11 +248,13 @@ void MonopdServer::delGame(Game *game, bool verbose)
 			// FIXME: DEPRECATED 1.0
 			if (verbose)
 				ioWrite(std::string("<monopd><updategamelist type=\"del\"><game id=\"") + itoa(game->id()) + "\"/></updategamelist></monopd>\n");
-			syslog( LOG_INFO, "del game: id=[%d], games=[%d]", game->id(), m_games.size() - 1 );
+			syslog( LOG_INFO, "del game: id=[%d], games=[%d]", game->id(), (int)m_games.size() - 1 );
 			m_games.erase(it);
 			break;
 		}
 	delete game;
+
+	updateSystemdStatus();
 }
 
 void MonopdServer::setGameDescription(Player *pInput, const std::string data)
@@ -277,16 +287,17 @@ Player *MonopdServer::newPlayer(Socket *socket, const std::string &name)
 	player->sendClientMsg();
 	sendGameList(player, true);
 
-	syslog( LOG_INFO, "new player: id=[%d], fd=[%d], name=[%s], players=[%d]", player->id(), socket->fd(), name.c_str(), m_players.size() );
+	syslog( LOG_INFO, "new player: id=[%d], fd=[%d], name=[%s], players=[%d]", player->id(), socket->fd(), name.c_str(), (int)m_players.size() );
 	player = 0;
 	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (player = *it) ; ++it)
-		printf("  player %16s %16s game %d bankrupt %d socket %d fd %d\n", player->name().c_str(), player->getStringProperty("host").c_str(), (player->game() ? player->game()->id() : -1), player->getBoolProperty("bankrupt"), player->socket(), player->socket() ? (int)player->socket()->fd() : -1);
+		printf("  player %16s %16s game %d bankrupt %d socket fd %d\n", player->name().c_str(), player->getStringProperty("host").c_str(), (player->game() ? player->game()->id() : -1), player->getBoolProperty("bankrupt"), player->socket() ? (int)player->socket()->fd() : -1);
 
 	// Re-register to meta server with updated player count.
 	registerMonopigator();
 	if (m_monopigatorEvent)
 		m_monopigatorEvent->setLaunchTime(time(0) + m_monopigatorEvent->frequency() );
 
+	updateSystemdStatus();
 	return player;
 }
 
@@ -348,8 +359,8 @@ void MonopdServer::delPlayer(Player *player)
 		if (*it == player)
 		{
 			removeFromScope(player);
-			syslog( LOG_INFO, "del player: id=[%d], fd=[%d], name=[%s], players=[%d]", player->id(), player->socket() ? player->socket()->fd() : 0, player->getStringProperty("name").c_str(), m_players.size() - 1 );
-			printf("delPlayer %d/%d\n", player->id(), m_players.size()-1);
+			syslog( LOG_INFO, "del player: id=[%d], fd=[%d], name=[%s], players=[%d]", player->id(), player->socket() ? player->socket()->fd() : 0, player->getStringProperty("name").c_str(), (int)m_players.size() - 1 );
+			printf("delPlayer %d/%d\n", player->id(), (int)m_players.size()-1);
 			delete player;
 			m_players.erase(it);
 			player = 0;
@@ -357,12 +368,14 @@ void MonopdServer::delPlayer(Player *player)
 		}
 
 	for(std::vector<Player *>::iterator it = m_players.begin(); it != m_players.end() && (player = *it) ; ++it)
-		printf("  player %16s %16s game %d bankrupt %d socket %d fd %d\n", player->name().c_str(), player->getStringProperty("host").c_str(), (player->game() ? player->game()->id() : -1), player->getBoolProperty("bankrupt"), player->socket(), player->socket() ? (int)player->socket()->fd() : -1);
+		printf("  player %16s %16s game %d bankrupt %d socket fd %d\n", player->name().c_str(), player->getStringProperty("host").c_str(), (player->game() ? player->game()->id() : -1), player->getBoolProperty("bankrupt"), player->socket() ? (int)player->socket()->fd() : -1);
 
 	// Re-register to meta server with updated player count.
 	registerMonopigator();
 	if (m_monopigatorEvent)
 		m_monopigatorEvent->setLaunchTime(time(0) + m_monopigatorEvent->frequency() );
+
+	updateSystemdStatus();
 }
 
 Player *MonopdServer::findPlayer(int playerId)
@@ -444,7 +457,7 @@ void MonopdServer::closedSocket(Socket *socket)
 	pInput->setSocket( 0 );
 	printf("%s socket 0 spec %d, bank %d, gamerun %d\n", pInput->name().c_str(), pInput->getBoolProperty("spectator"), pInput->getBoolProperty("bankrupt"), game->status() == Game::Run );
 	game->ioInfo("Connection with %s lost.", pInput->name().c_str());
-	
+
 	// Only remove from game when game not running, or when it's merely a spectator.
 	bool exitFromGame = false;
 	if (game->status() == Game::Run)
@@ -463,7 +476,7 @@ void MonopdServer::closedSocket(Socket *socket)
 	}
 	else
 		exitFromGame = true;
-	
+
 	if (exitFromGame)
 	{
 		printf("exit from game %d: %d\n", game->id(), pInput->id());
@@ -559,7 +572,7 @@ int MonopdServer::processEvents()
 			}
 			else
 				break;
-		} 
+		}
 	}
 	sendXMLUpdates();
 	return returnvalue;
@@ -569,7 +582,7 @@ void MonopdServer::registerMonopigator()
 {
 	int ircsock;
 	struct sockaddr_in sin;
-	struct hostent *hp = gethostbyname(m_gatorHost.c_str());	
+	struct hostent *hp = gethostbyname(m_gatorHost.c_str());
 	if (!hp)
 		return;
 
@@ -581,7 +594,7 @@ void MonopdServer::registerMonopigator()
 	if (connect(ircsock, (struct sockaddr *) & sin, sizeof(sin)))
 		return;
 
-	std::string getStr = std::string("GET /register.php?host=") + m_gatorIdentity + "&port=" + itoa(m_port) + "&version=" + escapeHTML(MONOPD_VERSION_STRING) + "&users=" + itoa(m_players.size()) + " HTTP/1.1\nHost:" + m_gatorHost + "\nUser-Agent: monopd/" + MONOPD_VERSION_STRING + "\n\n";
+	std::string getStr = std::string("GET /register.php?host=") + m_gatorIdentity + "&port=" + itoa(m_port) + "&version=" + escapeHTML(VERSION) + "&users=" + itoa(m_players.size()) + " HTTP/1.1\nHost:" + m_gatorHost + "\nUser-Agent: monopd/" + VERSION + "\n\n";
 	if (ircsock)
 	{
 		write(ircsock, getStr.c_str(), strlen(getStr.c_str()));
@@ -636,12 +649,16 @@ void MonopdServer::loadGameTemplates()
 	char str[256], *buf;
 	struct dirent *direntp;
 	std::string name = "", description = "";
-	
+
 	dirp = opendir(MONOPD_DATADIR "/games/");
 	if (!dirp)
 	{
 		syslog( LOG_ERR, "cannot open game directory, dir=[%s]", MONOPD_DATADIR "/games/" );
-		return;
+#if USE_SYSTEMD_DAEMON
+		sd_notifyf(1, "STATUS=Failed to start: cannot open game directory, dir=[%s]\nERRNO=%d", MONOPD_DATADIR "/games/", -1 );
+		usleep(100000);
+#endif /* USE_SYSTEMD_DAEMON */
+		exit(-1);
 	}
 	while ((direntp=readdir(dirp)) != NULL)
 	{
@@ -752,7 +769,7 @@ void MonopdServer::sendGameList(Player *pInput, const bool &sendTemplates)
 {
 	pInput->ioWrite("<monopd>");
 
-	// Supported game types for new games	
+	// Supported game types for new games
 	GameConfig *gcTmp = 0;
 	if (sendTemplates)
 		for(std::vector<GameConfig *>::iterator it = m_gameConfigs.begin() ; it != m_gameConfigs.end() && (gcTmp = *it) ; ++it )
@@ -770,7 +787,7 @@ void MonopdServer::sendGameList(Player *pInput, const bool &sendTemplates)
 	Game *gTmp = 0;
 	for(std::vector<Game *>::iterator it = m_games.begin(); it != m_games.end() && (gTmp = *it) ; ++it)
 		if (gTmp->status() == Game::Config)
-			pInput->ioWrite("<game id=\"%ld\" players=\"%d\" gametype=\"%s\" name=\"%s\" description=\"%s\" canbejoined=\"%d\"/>", gTmp->id(), gTmp->players(), gTmp->gameType().c_str(), gTmp->name().c_str(), gTmp->getStringProperty("description").c_str(), gTmp->getBoolProperty("canbejoined"));
+			pInput->ioWrite("<game id=\"%d\" players=\"%d\" gametype=\"%s\" name=\"%s\" description=\"%s\" canbejoined=\"%d\"/>", gTmp->id(), gTmp->players(), gTmp->gameType().c_str(), gTmp->name().c_str(), gTmp->getStringProperty("description").c_str(), gTmp->getBoolProperty("canbejoined"));
 
 	pInput->ioWrite("</updategamelist>");
 	// END FIXME: DEPRECATED 1.0
@@ -791,7 +808,7 @@ void MonopdServer::processInput(Socket *socket, const std::string data)
 			switch(data[1])
 			{
 			case 'n':
-				pNew = newPlayer(socket, data.substr(2));
+				pNew = newPlayer(socket, data.substr(2, 16));
 				sendXMLUpdates();
 				sendXMLUpdate(pNew, true, true); // give new player a full update (excluding self) so it knows who's in the lounge
 				return;
@@ -822,6 +839,13 @@ void MonopdServer::processInput(Socket *socket, const std::string data)
 	}
 }
 
+void MonopdServer::updateSystemdStatus() {
+#if USE_SYSTEMD_DAEMON
+	/* Update systemd status string */
+	sd_notifyf(0, "READY=1\nSTATUS=Running, %d players, %d games", (int)m_players.size(), (int)m_games.size() );
+#endif /* USE_SYSTEMD_DAEMON */
+}
+
 void MonopdServer::processCommands(Player *pInput, const std::string data2)
 {
 	char *data = (char *)data2.c_str();
@@ -832,13 +856,13 @@ void MonopdServer::processCommands(Player *pInput, const std::string data2)
 	switch(data[0])
 	{
 	case 'n':
-		setPlayerName(pInput, std::string(data+1));
+		setPlayerName(pInput, data2.substr(1, 16));
 		return;
 	case 'p':
 		switch(data[1])
 		{
 			case 'i':
-				pInput->setProperty("image", data+2, this);
+				pInput->setProperty("image", data2.substr(2, 32), this);
 				return;
 		}
 		break;
@@ -1066,7 +1090,7 @@ void MonopdServer::processCommands(Player *pInput, const std::string data2)
 	{
 		if(pInput->getBoolProperty("can_buyestate"))
 		{
-			switch(data[0]) 
+			switch(data[0])
 			{
 			case 'e':
 				switch(data[1])
@@ -1084,10 +1108,10 @@ void MonopdServer::processCommands(Player *pInput, const std::string data2)
 				break;
 			}
 		}
-		
+
 		if(pInput->getBoolProperty("jailed"))
 		{
-			switch(data[0]) 
+			switch(data[0])
 			{
 				case 'j':
 					switch(data[1])
@@ -1107,10 +1131,10 @@ void MonopdServer::processCommands(Player *pInput, const std::string data2)
 					break;
 			}
 		}
-		
+
 		if(pInput->getBoolProperty("can_roll"))
 		{
-			switch(data[0]) 
+			switch(data[0])
 			{
 				case 'r':
 					pInput->rollDice();
@@ -1118,12 +1142,12 @@ void MonopdServer::processCommands(Player *pInput, const std::string data2)
 					event->setLaunchTime(time(0) + 10);
 					return;
 			}
-			
+
 		}
 	}
 
 	// The following commands have their own availability checks.
-	switch(data[0])	
+	switch(data[0])
 	{
 		case 'E':
 			pInput->endTurn(true);
@@ -1132,7 +1156,7 @@ void MonopdServer::processCommands(Player *pInput, const std::string data2)
 			switch(data[1])
 			{
 				case 'd':
-					setGameDescription(pInput, data2.substr(2));
+					setGameDescription(pInput, data2.substr(2, 64));
 					return;
 				case 'c':
 					game->editConfiguration( pInput, data+2 );
@@ -1206,7 +1230,7 @@ void MonopdServer::sendXMLUpdate(Player *pOutput, bool fullUpdate, bool excludeS
 			{
 				if (updateEmpty)
 				{
-					pOutput->ioWrite("<monopd> ");
+					pOutput->ioWrite("<monopd>");
 					updateEmpty = false;
 				}
 				pOutput->ioWrite("%s", updateXML.c_str());
@@ -1224,7 +1248,7 @@ void MonopdServer::sendXMLUpdate(Player *pOutput, bool fullUpdate, bool excludeS
 		{
 			if (updateEmpty)
 			{
-				pOutput->ioWrite("<monopd> ");
+				pOutput->ioWrite("<monopd>");
 				updateEmpty = false;
 			}
 			pOutput->ioWrite("%s", updateXML.c_str());
@@ -1234,7 +1258,7 @@ void MonopdServer::sendXMLUpdate(Player *pOutput, bool fullUpdate, bool excludeS
 	// Let game handle updates *about* all of its objects ..
 	if (Game *game = pOutput->game())
 		updateEmpty = game->sendChildXMLUpdate(pOutput, updateEmpty);
-	
+
 	if (!updateEmpty)
 		pOutput->ioWrite("</monopd>\n");
 }
